'Program		Mandelbrot
'Version		1.11
'Language		FreeBASIC 0.18 for Linux

'Copyright © 2008-2009 David Watson [david@watzon.co.uk]

'Anyone is permitted to copy, modify, distribute and/or sell this program,
'provided that the copyright notice and permission notices appear in all copies.

'Anyone is permitted to use this program, provided that the user accepts that
'the program is provided "as is" and comes without any warranty.

'Anyone is permitted to integrate parts of the source code of this program into
'their own, provided that the author of this program is duly credited.



'######################### set up variables and screen #########################
dim as integer	b,c,g,i,mx,my,r,sx,sy,v,x,y
dim as double	cx,cy,px,py,tx,zm,zx,zy

screeninfo sx,sy				'what is current desktop resolution?

'want 3:2 window of 80% screen height
sy=sy*0.8
sx=sy*1.5

screenres sx,sy,8				'create window
windowtitle "Mandelbrot"

'middle of screen
mx=sx/2
my=sy/2

for c=0 to 255
	b=32+cos(c/40.7436)*31		'blue cosine sweeps once	/‾‾‾\___/
	g=32+cos(c/20.3718)*31		'green sweeps twice			/‾\_/‾\_/
	r=32+cos(c/10.1859)*31		'red sweeps four times		/\/\/\/\/
	palette c,(b shl 16)+(g shl 8)+r
next c

cx=-0.5							'initial view-centre real co-ord
cy=0							'initial view-centre imag co-ord
zm=sy/2.5						'initial zoom level
v=1								'set virgin flag



'################################## main loop ##################################
do
	for y=0 to sy-1
		for x=0 to sx-1
			'where is current pixel within mandelbrot?
			px=cx+(x-mx)/zm
			py=cy+(my-y)/zm

			zx=0
			zy=0

			'iterate the complex function z=z^2+c, where
			'zx is the real component of z,
			'zy is the imaginary component of z,
			'px is the real component of c and
			'py is the imaginary component of c
			for i=0 to 254
				tx=zx*zx-zy*zy+px
				zy=2*zx*zy+py
				zx=tx
				'test z to see if inside mandelbrot set
				if (zx*zx+zy*zy)>4 then exit for
			next i

			'if z fully outside set then i=0,
			'if outside but near boundary then i=1~254,
			'if inside then i=255
			pset (x,y),i
		next x
	next y

	if v=1 then					'if first screen refresh then print instructions
		color 42
		print "Left-click mouse to zoom in,"
		print "right-click to zoom out or"
		print "middle-click to reset view."
		print "Press any key to quit."
		v=0
	endif

	do
		getmouse x,y,,b
		if b=4 then				'middle button clicked
			cx=-0.5
			cy=0
			zm=sy/2.5
		elseif b=1 or b=2 then	'left or right button clicked
			'centre on pixel clicked with mouse
			cx=cx+(x-mx)/zm
			cy=cy+(my-y)/zm
			if b=1 then zm=zm*2 else zm=zm/2
		endif

		sleep 20
		if inkey<>"" then end
	loop until b>0
loop

